@@ -40,8 +40,15 @@ pub trait Example: 'static + Sized {
     fn required_features() -> wgpu::Features {
         wgpu::Features::empty()
     }
+    fn required_downlevel_capabilities() -> wgpu::DownlevelCapabilities {
+        wgpu::DownlevelCapabilities {
+            flags: wgpu::DownlevelFlags::empty(),
+            shader_model: wgpu::ShaderModel::Sm5,
+            ..wgpu::DownlevelCapabilities::default()
+        }
+    }
     fn required_limits() -> wgpu::Limits {
-        wgpu::Limits::downlevel_defaults() // These downlevel limits will allow the code to run on all possible hardware
+        wgpu::Limits::downlevel_webgl2_defaults() // These downlevel limits will allow the code to run on all possible hardware
     }
     fn init(
         config: &wgpu::SurfaceConfiguration,
@@ -95,7 +102,12 @@ async fn setup<E: Example>(title: &str) -> Setup {
     #[cfg(target_arch = "wasm32")]
     {
         use winit::platform::web::WindowExtWebSys;
-        console_log::init().expect("could not initialize logger");
+        let query_string = web_sys::window().unwrap().location().search().unwrap();
+        let level: log::Level = parse_url_query_string(&query_string, "RUST_LOG")
+            .map(|x| x.parse().ok())
+            .flatten()
+            .unwrap_or(log::Level::Error);
+        console_log::init_with_level(level).expect("could not initialize logger");
         std::panic::set_hook(Box::new(console_error_panic_hook::hook));
         // On wasm, append the canvas to the document body
         web_sys::window()
@@ -110,7 +122,7 @@ async fn setup<E: Example>(title: &str) -> Setup {
 
     log::info!("Initializing the surface...");
 
-    let backend = wgpu::util::backend_bits_from_env().unwrap_or(wgpu::Backends::PRIMARY);
+    let backend = wgpu::util::backend_bits_from_env().unwrap_or_else(wgpu::Backends::all);
 
     let instance = wgpu::Instance::new(backend);
     let (size, surface) = unsafe {
@@ -118,9 +130,10 @@ async fn setup<E: Example>(title: &str) -> Setup {
         let surface = instance.create_surface(&window);
         (size, surface)
     };
-    let adapter = wgpu::util::initialize_adapter_from_env_or_default(&instance, backend)
-        .await
-        .expect("No suitable GPU adapters found on the system!");
+    let adapter =
+        wgpu::util::initialize_adapter_from_env_or_default(&instance, backend, Some(&surface))
+            .await
+            .expect("No suitable GPU adapters found on the system!");
 
     #[cfg(not(target_arch = "wasm32"))]
     {
@@ -137,6 +150,21 @@ async fn setup<E: Example>(title: &str) -> Setup {
         required_features - adapter_features
     );
 
+    let required_downlevel_capabilities = E::required_downlevel_capabilities();
+    let downlevel_capabilities = adapter.get_downlevel_properties();
+    assert!(
+        downlevel_capabilities.shader_model >= required_downlevel_capabilities.shader_model,
+        "Adapter does not support the minimum shader model required to run this example: {:?}",
+        required_downlevel_capabilities.shader_model
+    );
+    assert!(
+        downlevel_capabilities
+            .flags
+            .contains(required_downlevel_capabilities.flags),
+        "Adapter does not support the downlevel capabilities required to run this example: {:?}",
+        required_downlevel_capabilities.flags - downlevel_capabilities.flags
+    );
+
     // Make sure we use the texture resolution limits from the adapter, so we can support images the size of the surface.
     let needed_limits = E::required_limits().using_resolution(adapter.limits());
 
@@ -233,7 +261,12 @@ fn start<E: Example>(
                 window.request_redraw();
             }
             event::Event::WindowEvent {
-                event: WindowEvent::Resized(size),
+                event:
+                    WindowEvent::Resized(size)
+                    | WindowEvent::ScaleFactorChanged {
+                        new_inner_size: &mut size,
+                        ..
+                    },
                 ..
             } => {
                 log::info!("Resizing to {:?}", size);
@@ -287,21 +320,22 @@ fn start<E: Example>(
                     }
                 }
 
-                let frame = match surface.get_current_frame() {
+                let frame = match surface.get_current_texture() {
                     Ok(frame) => frame,
                     Err(_) => {
                         surface.configure(&device, &config);
                         surface
-                            .get_current_frame()
+                            .get_current_texture()
                             .expect("Failed to acquire next surface texture!")
                     }
                 };
                 let view = frame
-                    .output
                     .texture
                     .create_view(&wgpu::TextureViewDescriptor::default());
 
                 example.render(&view, &device, &queue, &spawner);
+
+                frame.present();
             }
             _ => {}
         }
@@ -382,6 +416,25 @@ pub fn run<E: Example>(title: &str) {
     });
 }
 
+#[cfg(target_arch = "wasm32")]
+/// Parse the query string as returned by `web_sys::window()?.location().search()?` and get a
+/// specific key out of it.
+pub fn parse_url_query_string<'a>(query: &'a str, search_key: &str) -> Option<&'a str> {
+    let query_string = query.strip_prefix('?')?;
+
+    for pair in query_string.split('&') {
+        let mut pair = pair.split('=');
+        let key = pair.next()?;
+        let value = pair.next()?;
+
+        if key == search_key {
+            return Some(value);
+        }
+    }
+
+    None
+}
+
 #[cfg(test)]
 pub struct FrameworkRefTest {
     pub image_path: &'static str,
@@ -401,12 +454,9 @@ pub fn test<E: Example>(mut params: FrameworkRefTest) {
     assert_eq!(params.width % 64, 0, "width needs to be aligned 64");
 
     let features = E::required_features() | params.optional_features;
-    let limits = E::required_limits();
 
     test_common::initialize_test(
-        mem::take(&mut params.base_test_parameters)
-            .features(features)
-            .limits(limits),
+        mem::take(&mut params.base_test_parameters).features(features),
         |ctx| {
             let spawner = Spawner::new();
 
