@@ -38,6 +38,7 @@ impl fmt::Debug for Context {
 }
 
 impl Context {
+    #[cfg(not(target_arch = "wasm32"))]
     pub unsafe fn from_hal_instance<A: wgc::hub::HalApi>(hal_instance: A::Instance) -> Self {
         Self(wgc::hub::Global::from_hal_instance::<A>(
             "wgpu",
@@ -50,6 +51,7 @@ impl Context {
         &self.0
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
     pub fn enumerate_adapters(&self, backends: wgt::Backends) -> Vec<wgc::id::AdapterId> {
         self.0
             .enumerate_adapters(wgc::instance::AdapterInputs::Mask(backends, |_| {
@@ -57,6 +59,7 @@ impl Context {
             }))
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
     pub unsafe fn create_adapter_from_hal<A: wgc::hub::HalApi>(
         &self,
         hal_adapter: hal::ExposedAdapter<A>,
@@ -64,6 +67,7 @@ impl Context {
         self.0.create_adapter_from_hal(hal_adapter, PhantomData)
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
     pub unsafe fn create_device_from_hal<A: wgc::hub::HalApi>(
         &self,
         adapter: &wgc::id::AdapterId,
@@ -90,6 +94,7 @@ impl Context {
         Ok((device, device_id))
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
     pub unsafe fn create_texture_from_hal<A: wgc::hub::HalApi>(
         &self,
         hal_texture: A::Texture,
@@ -118,6 +123,17 @@ impl Context {
         }
     }
 
+    #[cfg(not(target_arch = "wasm32"))]
+    pub unsafe fn texture_as_hal<A: wgc::hub::HalApi, F: FnOnce(Option<&A::Texture>)>(
+        &self,
+        texture: &Texture,
+        hal_texture_callback: F,
+    ) {
+        self.0
+            .texture_as_hal::<A, F>(texture.id, hal_texture_callback)
+    }
+
+    #[cfg(not(target_arch = "wasm32"))]
     pub fn generate_report(&self) -> wgc::hub::GlobalReport {
         self.0.generate_report()
     }
@@ -768,6 +784,7 @@ impl crate::Context for Context {
         let id = self.0.request_adapter(
             &wgc::instance::RequestAdapterOptions {
                 power_preference: options.power_preference,
+                force_fallback_adapter: options.force_fallback_adapter,
                 compatible_surface: options.compatible_surface.map(|surface| surface.id.id),
             },
             wgc::instance::AdapterInputs::Mask(wgt::Backends::all(), |_| PhantomData),
@@ -934,6 +951,18 @@ impl crate::Context for Context {
         }
     }
 
+    fn surface_texture_discard(
+        &self,
+        texture: &Self::TextureId,
+        detail: &Self::SurfaceOutputDetail,
+    ) {
+        let global = &self.0;
+        match wgc::gfx_select!(texture.id => global.surface_texture_discard(detail.surface_id)) {
+            Ok(_status) => (),
+            Err(err) => self.handle_error_fatal(err, "Surface::discard_texture"),
+        }
+    }
+
     fn device_features(&self, device: &Self::DeviceId) -> Features {
         let global = &self.0;
         match wgc::gfx_select!(device.id => global.device_features(device.id)) {
@@ -962,10 +991,12 @@ impl crate::Context for Context {
         &self,
         device: &Self::DeviceId,
         desc: &ShaderModuleDescriptor,
+        shader_bound_checks: wgt::ShaderBoundChecks,
     ) -> Self::ShaderModuleId {
         let global = &self.0;
         let descriptor = wgc::pipeline::ShaderModuleDescriptor {
             label: desc.label.map(Borrowed),
+            shader_bound_checks,
         };
         let source = match desc.source {
             #[cfg(feature = "spirv")]
@@ -974,7 +1005,7 @@ impl crate::Context for Context {
                 let options = naga::front::spv::Options {
                     adjust_coordinate_space: false, // we require NDC_Y_UP feature
                     strict_capabilities: true,
-                    flow_graph_dump_prefix: None,
+                    block_ctx_dump_prefix: None,
                 };
                 let parser = naga::front::spv::Parser::new(spv.iter().cloned(), &options);
                 let module = parser.parse().unwrap();
@@ -1005,6 +1036,9 @@ impl crate::Context for Context {
         let global = &self.0;
         let descriptor = wgc::pipeline::ShaderModuleDescriptor {
             label: desc.label.map(Borrowed),
+            // Doesn't matter the value since spirv shaders aren't mutated to include
+            // runtime checks
+            shader_bound_checks: wgt::ShaderBoundChecks::unchecked(),
         };
         let (id, error) = wgc::gfx_select!(
             device.id => global.device_create_shader_module_spirv(device.id, &descriptor, Borrowed(&desc.source), PhantomData)
@@ -1145,17 +1179,17 @@ impl crate::Context for Context {
         // Limit is always less or equal to hal::MAX_BIND_GROUPS, so this is always right
         // Guards following ArrayVec
         assert!(
-            desc.bind_group_layouts.len() <= hal::MAX_BIND_GROUPS,
+            desc.bind_group_layouts.len() <= wgc::MAX_BIND_GROUPS,
             "Bind group layout count {} exceeds device bind group limit {}",
             desc.bind_group_layouts.len(),
-            hal::MAX_BIND_GROUPS
+            wgc::MAX_BIND_GROUPS
         );
 
         let temp_layouts = desc
             .bind_group_layouts
             .iter()
             .map(|bgl| bgl.id)
-            .collect::<ArrayVec<_, { hal::MAX_BIND_GROUPS }>>();
+            .collect::<ArrayVec<_, { wgc::MAX_BIND_GROUPS }>>();
         let descriptor = wgc::binding_model::PipelineLayoutDescriptor {
             label: desc.label.map(Borrowed),
             bind_group_layouts: Borrowed(&temp_layouts),
@@ -1187,7 +1221,7 @@ impl crate::Context for Context {
     ) -> Self::RenderPipelineId {
         use wgc::pipeline as pipe;
 
-        let vertex_buffers: ArrayVec<_, { hal::MAX_VERTEX_BUFFERS }> = desc
+        let vertex_buffers: ArrayVec<_, { wgc::MAX_VERTEX_BUFFERS }> = desc
             .vertex
             .buffers
             .iter()
@@ -1202,7 +1236,7 @@ impl crate::Context for Context {
             Some(_) => None,
             None => Some(wgc::device::ImplicitPipelineIds {
                 root_id: PhantomData,
-                group_ids: &[PhantomData; hal::MAX_BIND_GROUPS],
+                group_ids: &[PhantomData; wgc::MAX_BIND_GROUPS],
             }),
         };
         let descriptor = pipe::RenderPipelineDescriptor {
@@ -1261,7 +1295,7 @@ impl crate::Context for Context {
             Some(_) => None,
             None => Some(wgc::device::ImplicitPipelineIds {
                 root_id: PhantomData,
-                group_ids: &[PhantomData; hal::MAX_BIND_GROUPS],
+                group_ids: &[PhantomData; wgc::MAX_BIND_GROUPS],
             }),
         };
         let descriptor = pipe::ComputePipelineDescriptor {
@@ -1453,6 +1487,7 @@ impl crate::Context for Context {
         }
     }
 
+    #[cfg_attr(target_arch = "wasm32", allow(unused))]
     fn device_drop(&self, device: &Self::DeviceId) {
         #[cfg(not(target_arch = "wasm32"))]
         {
@@ -1889,7 +1924,7 @@ impl crate::Context for Context {
                 resolve_target: ca.resolve_target.map(|rt| rt.id),
                 channel: map_pass_channel(Some(&ca.ops)),
             })
-            .collect::<ArrayVec<_, { hal::MAX_COLOR_TARGETS }>>();
+            .collect::<ArrayVec<_, { wgc::MAX_COLOR_TARGETS }>>();
 
         let depth_stencil = desc.depth_stencil_attachment.as_ref().map(|dsa| {
             wgc::command::RenderPassDepthStencilAttachment {
@@ -1941,19 +1976,19 @@ impl crate::Context for Context {
         id
     }
 
-    fn command_encoder_clear_image(
+    fn command_encoder_clear_texture(
         &self,
         encoder: &Self::CommandEncoderId,
         texture: &crate::Texture,
         subresource_range: &wgt::ImageSubresourceRange,
     ) {
         let global = &self.0;
-        if let Err(cause) = wgc::gfx_select!(encoder.id => global.command_encoder_clear_image(
+        if let Err(cause) = wgc::gfx_select!(encoder.id => global.command_encoder_clear_texture(
             encoder.id,
             texture.id.id,
             subresource_range
         )) {
-            self.handle_error_nolabel(&encoder.error_sink, cause, "CommandEncoder::clear_image");
+            self.handle_error_nolabel(&encoder.error_sink, cause, "CommandEncoder::clear_texture");
         }
     }
 
